<?php

/*
Plugin Name: 		OxaPay Payment Method for Gravity Forms
Plugin URI: 		https://app.oxapay.com/signin
Description:		OxaPay is a leading crypto payment gateway that enables businesses to accept payments in various crypto currencies securely.
Version: 			1.0.0
Author: 			OxaPay.com
Author URI: 		https://OxaPay.com
*/

add_action('parse_request', array("GFOxaPay", "process_ipn"));
add_action('wp', array('GFOxaPay', 'maybe_thankyou_page'), 5);

add_action('init', array('GFOxaPay', 'init'));
register_activation_hook( __FILE__, array("GFOxaPay", "add_permissions"));

class GFOxaPay {

	private static $slug = "gravityformsoxapay";
	private static $version = "1.8";
	private static $min_gravityforms_version = "1.6.4";
	private static $supported_fields = array("checkbox", "radio", "select", "text", "website", "textarea", "email", "hidden", "number", "phone", "multiselect", "post_title", "post_tags", "post_custom_field", "post_content", "post_excerpt");

	public static function init() {
		add_filter("gform_logging_supported", array("GFOxaPay", "set_logging_supported"));
		if(basename($_SERVER['PHP_SELF']) == "plugins.php") {
			load_plugin_textdomain('gravityformsoxapay', FALSE, '/gravityformsoxapay/languages');
		}
		if(!self::is_gravityforms_supported()) {
			return;
		}
		if(is_admin()) {
			load_plugin_textdomain('gravityformsoxapay', FALSE, '/gravityformsoxapay/languages');
			if(function_exists('members_get_capabilities')) {
				add_filter('members_get_capabilities', array("GFOxaPay", "members_get_capabilities"));
			}
			add_filter("gform_addon_navigation", array('GFOxaPay', 'create_menu'));
			add_action('gform_payment_status', array('GFOxaPay','admin_edit_payment_status'), 3, 3);
			add_action('gform_entry_info', array('GFOxaPay','admin_edit_payment_status_details'), 4, 2);
			add_action('gform_after_update_entry', array('GFOxaPay','admin_update_payment'), 4, 2);
			if(self::is_oxapay_page()) {
				require_once(GFCommon::get_base_path() . "/tooltips.php");
				add_filter('gform_tooltips', array('GFOxaPay', 'tooltips'));
				wp_enqueue_script(array("sack"));
				require_once(self::get_base_path() . "/data.php");
				self::setup();
			} else if(in_array(RG_CURRENT_PAGE, array("admin-ajax.php"))) {
				require_once(self::get_base_path() . "/data.php");
				add_action('wp_ajax_gf_oxapay_update_feed_active', array('GFOxaPay', 'update_feed_active'));
				add_action('wp_ajax_gf_select_oxapay_form', array('GFOxaPay', 'select_oxapay_form'));
				add_action('wp_ajax_gf_oxapay_confirm_settings1', array('GFOxaPay', 'confirm_settings1'));
				add_action('wp_ajax_gf_oxapay_confirm_settings2', array('GFOxaPay', 'confirm_settings2'));
				add_action('wp_ajax_gf_oxapay_load_notifications', array('GFOxaPay', 'load_notifications'));
			} else if(RGForms::get("page") == "gf_settings") {
				RGForms::add_settings_page("OxaPay", array("GFOxaPay", "settings_page"), self::get_base_url() . "/images/oxapay_wordpress_icon_32.png");
			}
			add_filter('plugin_action_links_' . plugin_basename(__FILE__), function($links) {
				return array_merge(array('settings' => '<a href="'.admin_url('admin.php?page=gf_oxapay').'">'.__('Settings', 'gravityformsoxapay').'</a>'), $links);
			});
		} else {
			require_once(self::get_base_path() . "/data.php");
			add_filter("gform_confirmation", array("GFOxaPay", "send_to_oxapay"), 1000, 4);
			add_filter("gform_disable_post_creation", array("GFOxaPay", "delay_post"), 10, 3);
		}
	}

	public static function update_feed_active() {
		check_ajax_referer('gf_oxapay_update_feed_active','gf_oxapay_update_feed_active');
		$id = $_POST["feed_id"];
		$feed = GFOxaPayData::get_feed($id);
		GFOxaPayData::update_feed($id, $feed["form_id"], $_POST["is_active"], $feed["meta"]);
	}

	private static function get_key() {
		if(self::is_gravityforms_supported())
			return GFCommon::get_key();
		else
			return "";
	}

	public static function create_menu($menus) {
		$permission = self::has_access("gravityforms_oxapay");
		if(!empty($permission))
			$menus[] = array("name" => "gf_oxapay", "label" => __("OxaPay", "gravityformsoxapay"), "callback" => array("GFOxaPay", "oxapay_page"), "permission" => $permission);
		return $menus;
	}

	private static function setup() {
		if(get_option("gf_oxapay_version") != self::$version)
			GFOxaPayData::update_table();
		update_option("gf_oxapay_version", self::$version);
	}

	public static function tooltips($tooltips) {
		$oxapay_tooltips = array(
			"oxapay_transaction_type" => "<h6>".__("Transaction Type", "gravityformsoxapay")."</h6>",
			"oxapay_gravity_form" => "<h6>".__("Form", "gravityformsoxapay")."</h6>",
			"oxapay_options" => "<h6>".__("Options", "gravityformsoxapay")."</h6>",
			"oxapay_edit_payment_amount" => "<h6>".__("Payment Amount", "gravityformsoxapay")."</h6>",
			"oxapay_edit_payment_date" => "<h6>".__("Payment Date", "gravityformsoxapay")."</h6>",
			"oxapay_edit_payment_transaction_id" => "<h6>".__("Payment Transaction ID", "gravityformsoxapay")."</h6>",
			"oxapay_edit_payment_status" => "<h6>".__("Payment Status", "gravityformsoxapay")."</h6>"
		);
		return array_merge($tooltips, $oxapay_tooltips);
	}

	public static function delay_post($is_disabled, $form, $lead) {
		require_once(self::get_base_path() . "/data.php");
		$config = GFOxaPayData::get_feed_by_form($form["id"]);
		if(!$config)
			return $is_disabled;
		$config = $config[0];
		if(!self::has_oxapay_condition($form, $config))
			return $is_disabled;
		return $config["meta"]["delay_post"] == true;
	}




	public static function oxapay_page() {
		$view = rgget("view");
		if($view == "edit")
			self::edit_page(rgget("id"));
		else
			self::list_page();
	}

	private static function list_page() {
		if(!self::is_gravityforms_supported()){
			die(__(sprintf("OxaPay Add-On requires Gravity Forms %s. Upgrade automatically on the %sPlugin page%s.", self::$min_gravityforms_version, "<a href='plugins.php'>", "</a>"), "gravityformsoxapay"));
		}
		?>
	<div class="wrap" style="padding: 20px;">

    <?php 
    		if(!self::is_gravityforms_supported()){
			die(__(sprintf("OxaPay Add-On requires Gravity Forms %s. Upgrade automatically on the %sPlugin page%s.", self::$min_gravityforms_version, "<a href='plugins.php'>", "</a>"), "gravityformsoxapay"));
		}
		if(rgpost('action') == "delete"){
			check_admin_referer("list_action", "gf_oxapay_list");
			$id = absint($_POST["action_argument"]);
			GFOxaPayData::delete_feed($id);
			?>
			<?php
                echo "<script>
                    alert('" . __("Feed Deleted", "gravityformsoxapay") . "');
                </script>";

		} else if (!empty($_POST["bulk_action"])) {
			check_admin_referer("list_action", "gf_oxapay_list");
			$selected_feeds = $_POST["feed"];
			if(is_array($selected_feeds)) {
				foreach($selected_feeds as $feed_id)
					GFOxaPayData::delete_feed($feed_id);
			}
			?>
			<?php
                echo "<script>
                    alert('" . __("Feed Deleted", "gravityformsoxapay") . "');
                </script>";
			
		}

    ?>
    <div style="display: flex; align-items: center; margin-bottom: 20px;">
        <img src="<?php echo self::get_base_url(); ?>/images/oxapay_wordpress_icon_32.png" 
             style="margin-right: 15px;" 
             alt="OxaPay Icon" />
        <h2 style="margin: 0; font-size: 24px; color: #333;">
            <?php _e("OxaPay Forms", "gravityformsoxapay"); ?>
        </h2>
        <a href="admin.php?page=gf_oxapay&view=edit&id=0" 
           class="button" 
           style="margin-left: 15px; background-color: #3e7da6; color: white; border: none; padding: 10px 20px; font-size: 14px; box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1); text-decoration: none; transition: all 0.3s;"
           onmouseover="this.style.boxShadow='0 6px 8px rgba(0, 0, 0, 0.2)';"
           onmouseout="this.style.boxShadow='0 4px 6px rgba(0, 0, 0, 0.1)';">
            <?php _e("Add New", "gravityformsoxapay"); ?>
        </a>
    </div>

    <form id="feed_form" method="post">
        <?php wp_nonce_field('list_action', 'gf_oxapay_list') ?>
        <input type="hidden" id="action" name="action" />
        <input type="hidden" id="action_argument" name="action_argument" />

        <div style="display: flex; align-items: center; margin-bottom: 20px;">
            <select name="bulk_action" id="bulk_action" 
                    style="flex: 1; padding: 10px; font-size: 14px; border: 1px solid #ccc; border-radius: 4px;">
                <option value=""><?php _e("Bulk action", "gravityformsoxapay"); ?></option>
                <option value="delete"><?php _e("Delete", "gravityformsoxapay"); ?></option>
            </select>
            <input type="submit" class="button" 
                   value="<?php _e("Apply", "gravityformsoxapay"); ?>" 
                   style="margin-left: 10px; padding: 10px 20px; font-size: 14px;" 
                   onclick="if (jQuery('#bulk_action').val() === 'delete' && !confirm('<?php _e("Delete selected feeds?", "gravityformsoxapay"); ?>')) return false;" />
        </div>

		<table class="widefat fixed" cellspacing="0" style="margin-top: 20px; border-collapse: collapse;">
			<thead>
				<tr>
					<th style="width: 5%; text-align: center;">
						<input type="checkbox" id="select_all" onclick="toggleCheckboxes(this)" />
					</th>
					<?php
					$columns = [
						'Status' => 'status',
						'Title' => 'title',
						'ID' => 'id',
						'Transaction Type' => 'transaction_type'
					];
					foreach ($columns as $label => $field) :
						$is_current = isset($_GET['orderby']) && $_GET['orderby'] === $field;
						$order = isset($_GET['order']) && $_GET['order'] === 'desc' ? 'asc' : 'desc';
						$arrow = $is_current ? ($order === 'asc' ? '&#9650;' : '&#9660;') : '';
					?>
						<th style="text-align: <?php echo $field === "transaction_type" ? "center" : "left"; ?>;">
							<a href="#" style="color: #0073aa; text-decoration: none;">
								<?php echo $label; ?> <span><?php echo $arrow; ?></span>
							</a>
						</th>
					<?php endforeach; ?>
				</tr>
			</thead>
			<tfoot>
				<tr>
					<th style="width: 5%; text-align: center;">
						<input type="checkbox" id="select_all_footer" onclick="toggleCheckboxes(this)" />
					</th>
					<?php foreach ($columns as $label => $field) : ?>
						<th style="text-align: <?php echo $field === "transaction_type" ? "center" : "left"; ?>;">
							<a href="#" style="color: #0073aa; text-decoration: none;">
								<?php echo $label; ?> <span><?php echo $arrow; ?></span>
							</a>
						</th>
					<?php endforeach; ?>
				</tr>
			</tfoot>
			<tbody>
				<?php
				$settings = GFOxaPayData::get_feeds();
				if (is_array($settings) && sizeof($settings) > 0) :
					$row_count = 0;
					foreach ($settings as $setting) :
						$status = $setting["is_active"]
								? '<span style="color: green; font-weight: bold;">Active &#x2022;</span>'
								: '<span style="color: red; font-weight: bold;">Inactive &#x2022;</span>';
						$row_class = $row_count % 2 === 0 ? 'background-color: #f6f9fc;' : '';
						$row_count++;
					?>
						<tr style="<?php echo $row_class; ?>">
							<td style="text-align: center;"><input type="checkbox" name="feed[]" value="<?php echo $setting["id"] ?>" /></td>
							<td><?php echo $status; ?></td>
							<td>
								<a href="admin.php?page=gf_oxapay&view=edit&id=<?php echo $setting["id"] ?>" 
								style="font-weight: bold; color: #0073aa; text-decoration: none;">
									<?php echo $setting["form_title"]; ?>
								</a>
								<div class="row-actions">
								<span class="edit">
								<a href="admin.php?page=gf_oxapay&view=edit&id=<?php echo $setting["id"] ?>" ><?php _e("Edit", "gravityformsoxapay"); ?></a>
								|
								</span>
								<span class="view">
								<a href="admin.php?page=gf_entries&view=entries&id=<?php echo $setting["form_id"] ?>"><?php _e("Entries", "gravityformsoxapay"); ?></a>
								|
								</span>
								<span class="trash">
								<a href="javascript: if(confirm('Are you sure?')){ DeleteSetting(<?php echo $setting["id"] ?>);}"><?php _e("Delete", "gravityformsoxapay"); ?></a>
								</span>
							</div>

							</td>
							<td>
								<a href="admin.php?page=gf_entries&view=entries&id=<?php echo $setting["form_id"]; ?>" 
								style="color: #0073aa; text-decoration: none;">
									<?php echo $setting["form_id"]; ?>
								</a>
							</td>
							<td style="text-align: center;">
								<?php _e(ucfirst($setting["meta"]["type"]), "gravityformsoxapay"); ?>
							</td>
						</tr>
				<?php endforeach; else : ?>
					<tr>
						<td colspan="5" style="padding: 20px; text-align: center;"><?php _e("No feeds found.", "gravityformsoxapay"); ?></td>
					</tr>
				<?php endif; ?>
				</tbody>
			</table>
		</form>
		</div>	

		<script>
			function toggleCheckboxes(source) {
				const checkboxes = document.querySelectorAll('input[type="checkbox"]');
				checkboxes.forEach(checkbox => checkbox.checked = source.checked);
			}
		</script>


		<script type="text/javascript">
			function DeleteSetting(id){
				jQuery("#action_argument").val(id);
				jQuery("#action").val("delete");
				jQuery("#feed_form")[0].submit();
			}
			function ToggleActive(img, feed_id){
				var is_active = img.src.indexOf("active1.png") >=0
				if(is_active){
					img.src = img.src.replace("active1.png", "active0.png");
					jQuery(img).attr('title','<?php _e("Disbale", "gravityformsoxapay"); ?>').attr('alt', '<?php _e("Disable", "gravityformsoxapay"); ?>');
				}
				else{
					img.src = img.src.replace("active0.png", "active1.png");
					jQuery(img).attr('title','<?php _e("Enable", "gravityformsoxapay"); ?>').attr('alt', '<?php _e("Enable", "gravityformsoxapay"); ?>');
				}

				var mysack = new sack(ajaxurl);
				mysack.execute = 1;
				mysack.method = 'POST';
				mysack.setVar( "action", "gf_oxapay_update_feed_active" );
				mysack.setVar( "gf_oxapay_update_feed_active", "<?php echo wp_create_nonce("gf_oxapay_update_feed_active") ?>" );
				mysack.setVar( "feed_id", feed_id );
				mysack.setVar( "is_active", is_active ? 0 : 1 );
				mysack.onError = function() { alert('<?php _e("Error Updating field", "gravityformsoxapay"); ?>' )};
				mysack.runAJAX();
				return true;
			}
		</script>
		<?php
	}

	public static function load_notifications() {
		$form_id = $_POST["form_id"];
		$form = RGFormsModel::get_form_meta($form_id);
		$notifications = array();
		if(is_array(rgar($form, "notifications"))){
			foreach($form["notifications"] as $notification){
				$notifications[] = array("name" => $notification["name"], "id" => $notification["id"]);
			}
		}
		die(json_encode($notifications));
	}

	public static function confirm_settings1() {
		update_option("gf_oxapay_merchant", $_POST["n"]);
	}
    public static function confirm_settings2() {
        $lifetime = intval($_POST["n"]); 
        if ($lifetime < 15 || $lifetime > 2880) {
            $lifetime = 60;
        }
        update_option("gf_oxapay_lifetime", $lifetime); 
    }
    
	public static function settings_page() {
		if(rgpost("uninstall")) {
			check_admin_referer("uninstall", "gf_oxapay_uninstall");
			self::uninstall();
			?>
			<div class="updated fade" style="padding:20px;"><?php sprintf("OxaPay Payment Uninstalled. %s Reinstall again on Plugins Page %s", "<a href='plugins.php'>","</a>")?></div>
			<?php
			return;
		}
		?>
		<?php if (isset($_POST['gf_oxapay_save'])): ?>
        <?php
        // Save the settings
        if (isset($_POST['gf_oxapay_merchant'])) {
            update_option('gf_oxapay_merchant', sanitize_text_field($_POST['gf_oxapay_merchant']));
        }
        if (isset($_POST['gf_oxapay_lifetime'])) {
            update_option('gf_oxapay_lifetime', intval($_POST['gf_oxapay_lifetime']));
        }
        ?>
       
        <?php endif; ?>
        <form action="" method="post" style="max-width: 600px; margin: 0 auto; background: #f9f9f9; padding: 20px; border: 1px solid #ddd; border-radius: 8px; box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);">
            <?php wp_nonce_field("update", "gf_oxapay_update"); ?>
            
            <!-- Title Section -->
            <div style="text-align: center; margin-bottom: 20px;">
                <h1 style="font-size: 26px; font-weight: bold; color: #333; margin-bottom: 10px;">
                    <?php _e("OxaPay Payment Settings", "gravityformsoxapay"); ?>
                </h1>
                <img src="https://oxapay.com/media/logos/demo3.svg" 
                     alt="OxaPay Logo" 
                     style="width: 100%; max-width: 300px; height: auto; border-radius: 8px; margin-top: 10px;" />
            </div>
            
            <!-- OxaPay Merchant Key -->
            <div style="margin-bottom: 20px;">
                <label for="gf_oxapay_merchant" style="font-weight: bold; font-size: 14px; color: #333; margin-bottom: 8px; display: block;">
                    <?php _e("OxaPay Merchant Key", "gravityformsoxapay"); ?>
                </label>
                <input 
                    type="text" 
                    name="gf_oxapay_merchant" 
                    id="gf_oxapay_merchant" 
                    value="<?php echo esc_attr(get_option('gf_oxapay_merchant')); ?>" 
                    placeholder="Enter your merchant key"
                    style="width: 100%; padding: 10px; border: 1px solid #ccc; border-radius: 4px; font-size: 14px;" 
                />
                <p style="font-size: 12px; color: #666; margin-top: 5px;">
                    <?php _e('Find your OxaPay Merchant Key in your account settings. <a href="https://docs.oxapay.com/integrations/merchant-api" style="color: #0056b3; text-decoration: none;">Learn more</a>.', "gravityformsoxapay"); ?>
                </p>
            </div>
            
            <!-- Lifetime Input Field -->
            <div style="margin-bottom: 20px;">
                <label for="gf_oxapay_lifetime" style="font-weight: bold; font-size: 14px; color: #333; margin-bottom: 8px; display: block;">
                    <?php _e("Invoice Lifetime (in minutes)", "gravityformsoxapay"); ?>
                </label>
                <input 
                    type="number" 
                    name="gf_oxapay_lifetime" 
                    id="gf_oxapay_lifetime" 
                    min="15" 
                    max="2880" 
                    value="<?php echo esc_attr(get_option('gf_oxapay_lifetime', 60)); ?>" 
                    placeholder="Enter lifetime"
                    style="width: 100%; padding: 10px; border: 1px solid #ccc; border-radius: 4px; font-size: 14px;" 
                />
                <p style="font-size: 12px; color: #666; margin-top: 5px;">
                    <?php _e("Set the invoice lifetime in minutes (between 15 and 2880). Default is 60 minutes.", "gravityformsoxapay"); ?>
                </p>
            </div>
            
            <!-- Save Button -->
            <div style="text-align: center; margin-top: 20px;">
                <button 
                    type="submit" 
                    name="gf_oxapay_save" 
                    style="background-color: #0056b3; color: white; border: none; padding: 12px 25px; font-size: 16px; font-weight: bold; border-radius: 4px; cursor: pointer; transition: background-color 0.3s;">
                    <?php _e("Save Settings", "gravityformsoxapay"); ?>
                </button>
            </div>
        	<?php if (isset($_POST['gf_oxapay_save'])): ?>
                <p style="color: green; font-size: 14px; font-weight: bold; text-align: center; margin-top: 20px;">
                    <?php _e("Settings saved successfully!", "gravityformsoxapay"); ?>
                </p>
            <?php endif; ?>
        
        </form>


		<?php
	}

	private static function get_product_field_options($productFields, $selectedValue) {
		$options = "<option value=''>".__("Select", "gravityformsoxapay")."</option>";
		foreach($productFields as $field){
			$label = GFCommon::truncate_middle($field["label"], 30);
			$selected = $selectedValue == $field["id"] ? "selected='selected'" : "";
			$options .= "<option value='{$field["id"]}' {$selected}>{$label}</option>";
		}
		return $options;
	}


	// Edit Page
	private static function edit_page(){
		?>
		<style>
			#oxapay_submit_container{clear:both;}
			.oxapay_col_heading{padding-bottom:2px; border-bottom: 1px solid #ccc; font-weight:bold; width:120px;}
			.oxapay_field_cell {padding: 6px 17px 0 0; margin-right:15px;}
			.oxapay_validation_error{ background-color:#FFDFDF; margin-top:4px; margin-bottom:6px; padding-top:6px; padding-bottom:6px; border:1px dotted #C89797;}
			.oxapay_validation_error span {color: red;}
			.left_header{float:left; width:200px;}
			.margin_vertical_10{margin: 10px 0; padding-left:5px;}
			.margin_vertical_30{margin: 30px 0; padding-left:5px;}
			.width-1{width:300px;}
			.gf_oxapay_invalid_form{margin-top:30px; background-color:#FFEBE8;border:1px solid #CC0000; padding:10px; width:600px;}
		</style>
		<script type="text/javascript">
			var form = Array();
		</script>
		<div class="wrap">
			<!-- Title Section -->
			<!-- <h1 style="font-size: 24px; font-weight: bold; margin-bottom: 10px; text-align: left;">
				<?php _e("OxaPay Payment Settings", "gravityformsoxapay"); ?>
			</h1> -->
			<?php
			$id = !empty($_POST["oxapay_setting_id"]) ? $_POST["oxapay_setting_id"] : absint($_GET["id"]);
			$config = empty($id) ? array("meta" => array(), "is_active" => true) : GFOxaPayData::get_feed($id);
			$is_validation_error = false;

			$config["form_id"] = rgpost("gf_oxapay_submit") ? absint(rgpost("gf_oxapay_form")) : $config["form_id"];

			if (rgpost("gf_oxapay_submit")) {
				$config["meta"]["type"] = rgpost("gf_oxapay_type");
				$config["meta"]["delay_post"] = rgpost('gf_oxapay_delay_post');
				$config["meta"]["update_post_action"] = rgpost('gf_oxapay_update_action');

				$config = apply_filters('gform_oxapay_save_config', $config);
				$is_validation_error = apply_filters("gform_oxapay_config_validation", false, $config);

			}
			?>
        <form action="" method="post" style="max-width: 400px; margin: 50px auto 0 auto; background: #f9f9f9; padding: 20px; border: 1px solid #ddd; border-radius: 8px; box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);">
            <!-- Title Section -->
            <div style="text-align: center; margin-bottom: 20px;">
                <h1 style="font-size: 26px; font-weight: bold; color: #333; margin-bottom: 10px;">
                    <?php _e("OxaPay Payment Settings", "gravityformsoxapay"); ?>
                </h1>
                <img src="https://oxapay.com/media/logos/demo3.svg" 
                     alt="OxaPay Logo" 
                     style="width: 100%; max-width: 300px; height: auto; border-radius: 8px; margin-top: 10px;" />
            </div>

			<!-- <form method="post" action=""> -->
				<input type="hidden" name="oxapay_setting_id" value="<?php echo $id ?>" />

				<!-- Transaction Type Section -->
				<div style="margin-bottom: 20px;">
					<label for="gf_oxapay_type" style="font-weight: bold; font-size: 14px; color: #333; margin-bottom: 8px; display: block;">
						<?php _e("Transaction Type", "gravityformsoxapay"); ?>
					</label>
					<select id="gf_oxapay_type" name="gf_oxapay_type"style="width: 100%; padding: 10px; border: 1px solid #ccc; border-radius: 4px; font-size: 14px;"					>
						<option value="product" <?php echo rgar($config['meta'], 'type') == "product" ? "selected='selected'" : "" ?>><?php _e("Product", "gravityformsoxapay"); ?></option>
						<option value="donation" <?php echo rgar($config['meta'], 'type') == "donation" ? "selected='selected'" : "" ?>><?php _e("Donation", "gravityformsoxapay"); ?></option>
					</select>
					<p style="font-size: 12px; color: #666; margin-top: 5px;">
						<?php _e("Choose the type of transaction: either a product/service sale or a donation.", "gravityformsoxapay"); ?>
					</p>
				</div>

				<!-- Form Selection Section -->
				<div style="margin-bottom: 20px;">
					<label for="gf_oxapay_form" style="font-weight: bold; font-size: 14px; color: #333; margin-bottom: 8px; display: block;">
						<?php _e("Form", "gravityformsoxapay"); ?>
					</label>
					<select id="gf_oxapay_form" name="gf_oxapay_form" style="width: 100%; padding: 10px; border: 1px solid #ccc; border-radius: 4px; font-size: 14px;"  
>
						<option value=""><?php _e("Select Form", "gravityformsoxapay"); ?></option>
						<?php
						$available_forms = GFOxaPayData::get_available_forms(rgar($config, 'form_id'));
						foreach ($available_forms as $current_form) {
							$selected = absint($current_form->id) == rgar($config, 'form_id') ? 'selected="selected"' : '';
							?>
							<option value="<?php echo absint($current_form->id) ?>" <?php echo $selected; ?>><?php echo esc_html($current_form->title) ?></option>
							<?php
						}
						?>
					</select>
					<p style="font-size: 12px; color: #666; margin-top: 5px;">
						<?php _e("Select the form you want to associate with this transaction.", "gravityformsoxapay"); ?>
					</p>
				</div>

				<!-- Buttons Section -->
                <div style="margin-top: 30px; display: flex; justify-content: center; gap: 10px;">
                    <input type="submit" name="gf_oxapay_submit" 
                           value="<?php echo empty($id) ? __("Save", "gravityformsoxapay") : __("Update", "gravityformsoxapay"); ?>" 
                           class="button-primary" 
                           style="background-color: #0073aa; color: white; border: none; padding: 0 20px; font-size: 14px; border-radius: 4px; box-shadow: 0 2px 5px rgba(0, 0, 0, 0.1); cursor: pointer; transition: background-color 0.3s ease, transform 0.2s ease; height: 40px; width: 150px; display: flex; align-items: center; justify-content: center; text-align: center;"/>
                    
                    <input type="button" value="<?php _e("Cancel", "gravityformsoxapay"); ?>" 
                           class="button" 
                           onclick="javascript:document.location='admin.php?page=gf_oxapay'"
                           style="background-color: transparent; color: #0073aa; border: 2px solid #0073aa; padding: 0 20px; font-size: 14px; border-radius: 4px; box-shadow: 0 2px 5px rgba(0, 0, 0, 0.1); cursor: pointer; transition: background-color 0.3s ease, transform 0.2s ease; height: 40px; width: 150px; display: flex; align-items: center; justify-content: center; text-align: center;"/>
                </div>


				<?php
					if (!$is_validation_error && !empty($config["form_id"])) {
						$id = GFOxaPayData::update_feed($id, $config["form_id"], $config["is_active"], $config["meta"]);
						?>
                        <p style="color: green; font-size: 14px; font-weight: bold; text-align: center; margin-top: 20px;">
                            <?php 
                                echo __("Feed updated. ");
                                echo '<a href="?page=gf_oxapay" style="color: #0073aa; text-decoration: none;">Go back</a>';
                            ?>
                        </p>

						<?php
					} else {
						$is_validation_error = true;
					}
        	
				?>
			</form>
		</div>

		<script type="text/javascript">
			jQuery(document).ready(function(){
				SetPeriodNumber('#gf_oxapay_billing_cycle_number', jQuery("#gf_oxapay_billing_cycle_type").val());
				SetPeriodNumber('#gf_oxapay_trial_period_number', jQuery("#gf_oxapay_trial_period_type").val());
			});

			function SelectType(type){
				jQuery("#oxapay_field_group").slideUp();

				jQuery("#oxapay_field_group input[type=\"text\"], #oxapay_field_group select").val("");
				jQuery("#gf_oxapay_trial_period_type, #gf_oxapay_billing_cycle_type").val("M");

				jQuery("#oxapay_field_group input:checked").attr("checked", false);

				if(type){
					jQuery("#oxapay_form_container").slideDown();
					jQuery("#gf_oxapay_form").val("");
				}
				else{
					jQuery("#oxapay_form_container").slideUp();
				}
			}

			function SelectForm(type, formId, settingId){
				if(!formId){
					jQuery("#oxapay_field_group").slideUp();
					return;
				}

				jQuery("#oxapay_wait").show();
				jQuery("#oxapay_field_group").slideUp();

				var mysack = new sack(ajaxurl);
				mysack.execute = 1;
				mysack.method = 'POST';
				mysack.setVar( "action", "gf_select_oxapay_form" );
				mysack.setVar( "gf_select_oxapay_form", "<?php echo wp_create_nonce("gf_select_oxapay_form") ?>" );
				mysack.setVar( "type", type);
				mysack.setVar( "form_id", formId);
				mysack.setVar( "setting_id", settingId);
				mysack.onError = function() {jQuery("#oxapay_wait").hide(); alert('<?php _e("Error Selecting Form", "gravityformsoxapay"); ?>' )};
				mysack.runAJAX();

				return true;
			}

			function EndSelectForm(form_meta, customer_fields, recurring_amount_options){

				//setting global form object
				form = form_meta;

				var type = jQuery("#gf_oxapay_type").val();

				jQuery(".gf_oxapay_invalid_form").hide();
				if( (type == "product") && GetFieldsByType(["product"]).length == 0){
					jQuery("#gf_oxapay_invalid_product_form").show();
					jQuery("#oxapay_wait").hide();
					return;
				}
				else if(type == "donation" && GetFieldsByType(["product", "donation"]).length == 0){
					jQuery("#gf_oxapay_invalid_donation_form").show();
					jQuery("#oxapay_wait").hide();
					return;
				}

				jQuery(".oxapay_field_container").hide();
				jQuery("#oxapay_customer_fields").html(customer_fields);
				jQuery("#gf_oxapay_recurring_amount").html(recurring_amount_options);

				//displaying delayed post creation setting if current form has a post field
				var post_fields = GetFieldsByType(["post_title", "post_content", "post_excerpt", "post_category", "post_custom_field", "post_image", "post_tag"]);
				if(post_fields.length > 0){
					jQuery("#oxapay_post_action").show();
				}
				else{
					jQuery("#gf_oxapay_delay_post").attr("checked", false);
					jQuery("#oxapay_post_action").hide();
				}

				jQuery("#gf_oxapay_update_post").attr("checked", false);
				jQuery("#oxapay_post_update_action").hide();

				SetPeriodNumber('#gf_oxapay_billing_cycle_number', jQuery("#gf_oxapay_billing_cycle_type").val());
				SetPeriodNumber('#gf_oxapay_trial_period_number', jQuery("#gf_oxapay_trial_period_type").val());

				//Calling callback functions
				jQuery(document).trigger('oxapayFormSelected', [form]);

				jQuery("#gf_oxapay_conditional_enabled").attr('checked', false);
				SetOxaPayCondition("","");

				

				jQuery("#oxapay_field_container_" + type).show();
				jQuery("#oxapay_field_group").slideDown();
				jQuery("#oxapay_wait").hide();
			}

			function SetPeriodNumber(element, type){
				var prev = jQuery(element).val();

				var min = 1;
				var max = 0;
				switch(type){
					case "D" :
						max = 100;
					break;
					case "W" :
						max = 52;
					break;
					case "M" :
						max = 12;
					break;
					case "Y" :
						max = 5;
					break;
				}
				var str="";
				for(var i=min; i<=max; i++){
					var selected = prev == i ? "selected='selected'" : "";
					str += "<option value='" + i + "' " + selected + ">" + i + "</option>";
				}
				jQuery(element).html(str);
			}

			function GetFieldsByType(types){
				var fields = new Array();
				for(var i=0; i<form["fields"].length; i++){
					if(IndexOf(types, form["fields"][i]["type"]) >= 0)
						fields.push(form["fields"][i]);
				}
				return fields;
			}
			function IndexOf(ary, item){
				for(var i=0; i<ary.length; i++)
					if(ary[i] == item)
						return i;

				return -1;
			}
			<?php
			if(!empty($config["form_id"])){
				?>
				form = <?php echo GFCommon::json_encode($form)?> ;
				jQuery(document).ready(function(){
					var selectedField = "<?php echo str_replace('"', '\"', $config["meta"]["oxapay_conditional_field_id"])?>";
					var selectedValue = "<?php echo str_replace('"', '\"', $config["meta"]["oxapay_conditional_value"])?>";
					SetOxaPayCondition(selectedField, selectedValue);
				});
				<?php
			}
			?>
			function SetOxaPayCondition(selectedField, selectedValue){
				jQuery("#gf_oxapay_conditional_field_id").html(GetSelectableFields(selectedField, 20));
				var optinConditionField = jQuery("#gf_oxapay_conditional_field_id").val();
				var checked = jQuery("#gf_oxapay_conditional_enabled").attr('checked');
				if(optinConditionField){
					jQuery("#gf_oxapay_conditional_message").hide();
					jQuery("#gf_oxapay_conditional_fields").show();
					jQuery("#gf_oxapay_conditional_value_container").html(GetFieldValues(optinConditionField, selectedValue, 20));
					jQuery("#gf_oxapay_conditional_value").val(selectedValue);
				} else {
					jQuery("#gf_oxapay_conditional_message").show();
					jQuery("#gf_oxapay_conditional_fields").hide();
				}
				if(!checked) jQuery("#gf_oxapay_conditional_container").hide();
			}
			function GetFieldValues(fieldId, selectedValue, labelMaxCharacters){
				if(!fieldId)
					return "";
				var str = "";
				var field = GetFieldById(fieldId);
				if(!field)
					return "";
				var isAnySelected = false;
				if(field["type"] == "post_category" && field["displayAllCategories"]){
					str += '<?php $dd = wp_dropdown_categories(array("class"=>"optin_select", "orderby"=> "name", "id"=> "gf_oxapay_conditional_value", "name"=> "gf_oxapay_conditional_value", "hierarchical"=>true, "hide_empty"=>0, "echo"=>false)); echo str_replace("\n","", str_replace("'","\\'",$dd)); ?>';
				} else if(field.choices) {
					str += '<select id="gf_oxapay_conditional_value" name="gf_oxapay_conditional_value" class="optin_select">'
					for(var i=0; i<field.choices.length; i++){
						var fieldValue = field.choices[i].value ? field.choices[i].value : field.choices[i].text;
						var isSelected = fieldValue == selectedValue;
						var selected = isSelected ? "selected='selected'" : "";
						if(isSelected)
							isAnySelected = true;
						str += "<option value='" + fieldValue.replace(/'/g, "&#039;") + "' " + selected + ">" + TruncateMiddle(field.choices[i].text, labelMaxCharacters) + "</option>";
					}
					if(!isAnySelected && selectedValue){
						str += "<option value='" + selectedValue.replace(/'/g, "&#039;") + "' selected='selected'>" + TruncateMiddle(selectedValue, labelMaxCharacters) + "</option>";
					}
					str += "</select>";
				}
				else
				{
					selectedValue = selectedValue ? selectedValue.replace(/'/g, "&#039;") : "";
					//create a text field for fields that don't have choices (i.e text, textarea, number, email, etc...)
					str += "<input type='text' placeholder='<?php _e("Select a Value", "gravityformsoxapay"); ?>' id='gf_oxapay_conditional_value' name='gf_oxapay_conditional_value' value='" + selectedValue.replace(/'/g, "&#039;") + "'>";
				}
				return str;
			}
			function GetFieldById(fieldId){
				for(var i=0; i<form.fields.length; i++){
					if(form.fields[i].id == fieldId)
						return form.fields[i];
				}
				return null;
			}
			function TruncateMiddle(text, maxCharacters){
				if(!text)
					return "";
				if(text.length <= maxCharacters)
					return text;
				var middle = parseInt(maxCharacters / 2);
				return text.substr(0, middle) + "..." + text.substr(text.length - middle, middle);
			}
			function GetSelectableFields(selectedFieldId, labelMaxCharacters){
				var str = "";
				var inputType;
				for(var i=0; i<form.fields.length; i++){
					fieldLabel = form.fields[i].adminLabel ? form.fields[i].adminLabel : form.fields[i].label;
					inputType = form.fields[i].inputType ? form.fields[i].inputType : form.fields[i].type;
					if (IsConditionalLogicField(form.fields[i])) {
						var selected = form.fields[i].id == selectedFieldId ? "selected='selected'" : "";
						str += "<option value='" + form.fields[i].id + "' " + selected + ">" + TruncateMiddle(fieldLabel, labelMaxCharacters) + "</option>";
					}
				}
				return str;
			}
			function IsConditionalLogicField(field){
				inputType = field.inputType ? field.inputType : field.type;
				var supported_fields = ["checkbox", "radio", "select", "text", "website", "textarea", "email", "hidden", "number", "phone", "multiselect", "post_title",
										"post_tags", "post_custom_field", "post_content", "post_excerpt"];

				var index = jQuery.inArray(inputType, supported_fields);

				return index >= 0;
			}

		</script>

		<?php

	}

	public static function select_oxapay_form(){
		check_ajax_referer("gf_select_oxapay_form", "gf_select_oxapay_form");
		$type = $_POST["type"];
		$form_id = intval($_POST["form_id"]);
		$setting_id = intval($_POST["setting_id"]);
		$form = RGFormsModel::get_form_meta($form_id);
		$customer_fields = self::get_customer_information($form);
		$recurring_amount_fields = self::get_product_options($form, "");
		die("EndSelectForm(" . GFCommon::json_encode($form) . ", '" . str_replace("'", "\'", $customer_fields) . "', '" . str_replace("'", "\'", $recurring_amount_fields) . "');");
	}

	public static function add_permissions(){
		global $wp_roles;
		$wp_roles->add_cap("administrator", "gravityforms_oxapay");
		$wp_roles->add_cap("administrator", "gravityforms_oxapay_uninstall");
	}

	public static function members_get_capabilities( $caps ) {
		return array_merge($caps, array("gravityforms_oxapay", "gravityforms_oxapay_uninstall"));
	}

	public static function get_active_config($form){
		require_once(self::get_base_path() . "/data.php");
		$configs = GFOxaPayData::get_feed_by_form($form["id"], true);
		if(!$configs)
			return false;
		foreach($configs as $config){
			if(self::has_oxapay_condition($form, $config))
				return $config;
		}
		return false;
	}

	public static function send_to_oxapay($confirmation, $form, $entry, $ajax) {
		if(RGForms::post("gform_submit") != $form["id"]) {
			return $confirmation;
		}
		$config = self::get_active_config($form);
		if(!$config)
		{
			self::log_debug("NOT sending to OxaPay: No OxaPay setup was located for form_id = {$form['id']}.");
			return $confirmation;
		}
		gform_update_meta($entry["id"], "oxapay_feed_id", $config["id"]);
		gform_update_meta($entry["id"], "payment_gateway", "oxapay");
		$currency = GFCommon::get_currency();

		$amount = 0;
		switch($config["meta"]["type"]) {
			case "product" :
				$amount = self::get_product_query_string($form, $entry);
				break;
			case "donation" :
				$amount = self::get_donation_query_string($form, $entry);
				break;
		}

		if(!$amount) {
			self::log_debug("NOT sending to OxaPay: The price is either zero");
			return $confirmation;
		}
		$callbackUrl = trailingslashit(home_url()) . "?gf_oxapay_ipn=oxapay&fid={$form["id"]}&eid={$entry["id"]}";
		$body = array(
			'merchant'    => get_option("gf_oxapay_merchant"),
            'currency'    => 'USD',
            'lifeTime'    => get_option("gf_oxapay_lifetime"),
			'amount'      => floatval($amount),
			'description' => 'GF_Entry_'.$entry["id"],
			'orderId'     => strval($entry["id"]),
			'callbackUrl' => $callbackUrl,
			'returnUrl' => home_url(),

		);

		$header = array('Content-Type' => 'application/json', 'Accept' => 'application/json');
		$result = wp_remote_post($u = 'https://api.oxapay.com/merchants/request', $a = array('timeout' => 25, 'body' => wp_json_encode($body), 'headers' => $header, 'sslverify' => false));
		GFAPI::update_entry( $entry );
		if (is_wp_error($result))
		{
			$message = __('Erorr', 'gravityformsoxapay') . ' : ' . $result->get_error_message();
			self::log_debug($message);
			exit($message);
			return $confirmation;
		}
		$pay = json_decode(wp_remote_retrieve_body($result));
		if ($pay->result == 100)
		{
			gform_update_meta($entry["id"], "oxapay_trackid", $pay->trackId);
			$url = $pay->payLink;
		}
		else
		{
			$message = __('Erorr', 'gravityformsoxapay') . ' : ' . $pay->message;
			self::log_debug($message);
			exit($message);
			return $confirmation;
		}
		if(headers_sent() || $ajax) {
			$confirmation = "<script>function gformRedirect(){document.location.href='$url';}";
			if(!$ajax)
				$confirmation .="gformRedirect();";
			$confirmation .="</script>";
		} else {
			$confirmation = array("redirect" => $url);
		}
		return $confirmation;
	}

	public static function has_oxapay_condition($form, $config) {
		$config = $config["meta"];
		$operator = isset($config["oxapay_conditional_operator"]) ? $config["oxapay_conditional_operator"] : "";
		$field = RGFormsModel::get_field($form, $config["oxapay_conditional_field_id"]);
		if(empty($field) || !$config["oxapay_conditional_enabled"])
			return true;
		$is_visible = !RGFormsModel::is_field_hidden($form, $field, array());
		$field_value = RGFormsModel::get_field_value($field, array());
		$is_value_match = RGFormsModel::is_value_match($field_value, $config["oxapay_conditional_value"], $operator);
		$go_to_oxapay = $is_value_match && $is_visible;
		return $go_to_oxapay;
	}

	public static function get_config($form_id){
		if(!class_exists("GFOxaPayData"))
			require_once(self::get_base_path() . "/data.php");
		$config = GFOxaPayData::get_feed_by_form($form_id);
		if(!$config)
			return false;
		return $config[0]; //only one feed per form is supported (left for backwards compatibility)
	}

	public static function get_config_by_entry($entry) {
		if(!class_exists("GFOxaPayData"))
			require_once(self::get_base_path() . "/data.php");
		$feed_id = gform_get_meta($entry["id"], "oxapay_feed_id");
		$feed = GFOxaPayData::get_feed($feed_id);
		return !empty($feed) ? $feed : false;
	}

	public static function maybe_thankyou_page(){
		if(!self::is_gravityforms_supported())
			return;
		if($str = RGForms::get("gf_oxapay_return"))
		{
			$str = base64_decode($str);
			parse_str($str, $query);
			if(wp_hash("ids=" . $query["ids"]) == $query["hash"]){
				list($form_id, $lead_id) = explode("|", $query["ids"]);

				$form = RGFormsModel::get_form_meta($form_id);
				$lead = RGFormsModel::get_lead($lead_id);

				if(!class_exists("GFFormDisplay"))
					require_once(GFCommon::get_base_path() . "/form_display.php");

				$confirmation = GFFormDisplay::handle_confirmation($form, $lead, false);

				if(is_array($confirmation) && isset($confirmation["redirect"])){
					header("Location: {$confirmation["redirect"]}");
					exit;
				}
				if (isset($_REQUEST['res'])) {
					$confirmation .= '<br />'.__("TrackId", "gravityformsoxapay").' : '.$_REQUEST['res'];
				}

				GFFormDisplay::$submission[$form_id] = array("is_confirmation" => true, "confirmation_message" => $confirmation, "form" => $form, "lead" => $lead);

			}
		}
	}
    public static function convertToWPStatus($status)
    {
        switch ($status) {
            case 'Waiting':
                $result = 'Pending';
                break;

            case 'Confirming':
                $result = 'Processing';
                break;

            case 'Failed':
            case 'Expired':
                $result = 'Failed';
                break;
            case 'Paid':
                $result = 'Completed';
                break;
        }
        return $result;
    }


	public static function process_ipn($wp) {
		if (!isset($_GET['gf_oxapay_ipn'])) {
            return;
        }

		self::log_debug("IPN request received. Starting to process...");
		self::log_debug(print_r($_REQUEST, true));

		if (isset($_GET['fid'],$_GET['eid']))
		{
			global $current_user;
			$user_id = 0;
			$user_name = "System";
    		$form_id = sanitize_text_field($_GET['fid']);
    		$entry_id = sanitize_text_field($_GET['eid']);

			if($current_user && $user_data = get_userdata($current_user->ID)){
				$user_id = $current_user->ID;
				$user_name = $user_data->display_name;
			}

			$entry = RGFormsModel::get_lead($entry_id);

			if(!$entry) {
				$message = "Entry could not be found. Entry ID: {$entry_id}. Aborting.";
				self::log_error($message);
				echo $message;
				exit;
			}
			self::log_debug("Entry has been found." . print_r($entry, true));

			$config = self::get_config_by_entry($entry);
			if(!$config) {
				$message = "Form no longer is configured with OxaPay Addon. Form ID: {$entry["form_id"]}. Aborting.";
				self::log_error($message);
				echo $message;
				exit;
			}
			self::log_debug("Form {$entry["form_id"]} is properly configured.");

		
		    $postData = file_get_contents('php://input');
            $data = json_decode($postData, true);
            $apiSecretKey = get_option("gf_oxapay_merchant");
            $hmacHeader = $_SERVER['HTTP_HMAC'];
            $calculatedHmac = hash_hmac('sha512', $postData, $apiSecretKey);
            if ($calculatedHmac === $hmacHeader) {
    			$trackId = sanitize_text_field($data['trackId']);
    			$pay_trackid =  gform_get_meta($entry["id"], "oxapay_trackid");
    			if ($trackId != $pay_trackid)
    			{
    				$message = __('Error : Payment Data (TrackId) Mismatch', 'gravityformsoxapay');
    				self::log_error($message);
    				$entry["payment_status"] = "Failed";
    				GFAPI::update_entry($entry);
    				RGFormsModel::add_note($entry["id"], $user_id, $user_name, $message);
    				echo $message;
    				exit;
			    }
                $orderId = $data['orderId'];

				$amount = 0;
				$form = RGFormsModel::get_form_meta($form_id);
				switch($config["meta"]["type"]) {
					case "product" :
						$amount = self::get_product_query_string($form, $entry);
						break;
					case "donation" :
						$amount = self::get_donation_query_string($form, $entry);
						break;
				}
				
				self::log_debug("Setting payment status...");
				$payment_status = self::convertToWPStatus($data['status']);
				$entry["payment_status"] = $payment_status;
				$entry["payment_amount"] = $amount;
				$entry["payment_date"] = gmdate("y-m-d H:i:s");
				if ($payment_status == 'Completed') {
					$entry["transaction_id"] = $data['txID'];
				}
				else {
					$entry["transaction_id"] = $trackId;
				}
				$entry["transaction_type"] = 1; 
				self::log_debug("Updating entry.");
				GFAPI::update_entry($entry); 
				RGFormsModel::add_note($entry["id"], $user_id, $user_name, sprintf(__("Payment Accepted . Amount : %s . TrackId : %s", "gravityformsoxapay"), GFCommon::to_money($entry["payment_amount"], $entry["currency"]), $trackId));

				GFOxaPayData::insert_transaction($entry["id"], "payment", "", $trackId, $trackId, $amount);
				if ($payment_status == 'Completed') {
					do_action("gform_post_payment_status", $config, $entry, 'Completed', $trackId, 0, $amount, '', '');
				}
				header('location: '.add_query_arg(array("ref" => $trackId), self::return_url($form["id"], $entry["id"])));
				exit;
                http_response_code(200);
                echo 'OK';
            } else {
                http_response_code(400);
                echo 'Invalid HMAC signature';
            }
			}
		
		else
		{
			$message = __('System (Permission) Error!', 'gravityformsoxapay');
			self::log_error($message);
			echo $message;
			exit;
		}
	}

	public static function fulfill_order(&$entry, $transaction_id, $amount){
		$config = self::get_config_by_entry($entry);
		if(!$config){
			self::log_error("Order can't be fulfilled because feed wasn't found for form: {$entry["form_id"]}");
			return;
		}

		$form = RGFormsModel::get_form_meta($entry["form_id"]);
		if($config["meta"]["delay_post"]){
			self::log_debug("Creating post.");
			RGFormsModel::create_post($form, $entry);
		}

		self::log_debug("Before gform_oxapay_fulfillment.");
		do_action("gform_oxapay_fulfillment", $entry, $config, $transaction_id, $amount);
	}

	private static function get_product_query_string($form, $entry){
		$fields = "";
		$products = GFCommon::get_product_fields($form, $entry, true);
		$product_index = 1;
		$total = 0;
		$discount = 0;

		foreach($products["products"] as $product){
			$option_fields = "";
			$price = GFCommon::to_number($product["price"]);
			if(is_array(rgar($product,"options"))){
				$option_index = 1;
				foreach($product["options"] as $option){
					$field_label = urlencode($option["field_label"]);
					$option_name = urlencode($option["option_name"]);
					$option_fields .= "&on{$option_index}_{$product_index}={$field_label}&os{$option_index}_{$product_index}={$option_name}";
					$price += GFCommon::to_number($option["price"]);
					$option_index++;
				}
			}

			$name = urlencode($product["name"]);
			if($price > 0)
			{
				$fields .= "&item_name_{$product_index}={$name}&amount_{$product_index}={$price}&quantity_{$product_index}={$product["quantity"]}{$option_fields}";
				$total += $price * $product['quantity'];
				$product_index++;
			}
			else{
				$discount += abs($price) * $product['quantity'];
			}

		}

		if($discount > 0){
			$fields .= "&discount_amount_cart={$discount}";
		}

		$shipping = !empty($products["shipping"]["price"]) ? "&shipping_1={$products["shipping"]["price"]}" : "";
		$fields .= "{$shipping}&cmd=_cart&upload=1";

		return $total;
		return $total > 0 && $total > $discount ? $fields : false;
	}

	private static function get_donation_query_string($form, $entry){
		$fields = "";

		$donations = GFCommon::get_fields_by_type($form, array("donation"));
		$total = 0;
		$purpose = "";
		foreach($donations as $donation){
			$value = RGFormsModel::get_lead_field_value($entry, $donation);
			list($name, $price) = explode("|", $value);
			if(empty($price)){
				$price = $name;
				$name = $donation["label"];
			}
			$purpose .= $name . ", ";
			$price = GFCommon::to_number($price);
			$total += $price;
		}

		if($total == 0){
			$products = GFCommon::get_product_fields($form, $entry, true);
			foreach($products["products"] as $product){
				$options = "";
				if(is_array($product["options"]) && !empty($product["options"])){
					$options = " (";
					foreach($product["options"] as $option){
						$options .= $option["option_name"] . ", ";
					}
					$options = substr($options, 0, strlen($options)-2) . ")";
				}
				$quantity = GFCommon::to_number($product["quantity"]);
				$quantity_label = $quantity > 1 ? $quantity . " " : "";
				$purpose .= $quantity_label . $product["name"] . $options . ", ";
			}

			$total = GFCommon::get_order_total($form, $entry);
		}

		if(!empty($purpose))
			$purpose = substr($purpose, 0, strlen($purpose)-2);

		$purpose = urlencode($purpose);

		if(strlen($purpose) > 127)
			$purpose = substr($purpose, 0, 124) . "...";

		$fields = "&amount={$total}&item_name={$purpose}&cmd=_donations";

		return $total;
		return $total > 0 ? $fields : false;
	}

	public static function uninstall(){
		require_once(self::get_base_path() . "/data.php");
		if(!GFOxaPay::has_access("gravityforms_oxapay_uninstall"))
			die(__("permission denied", "gravityformsoxapay"));
		GFOxaPayData::drop_tables();
		delete_option("gf_oxapay_site_name");
		delete_option("gf_oxapay_auth_token");
		delete_option("gf_oxapay_version");
		$plugin = "gravityformsoxapay/index.php";
		deactivate_plugins($plugin);
		update_option('recently_activated', array($plugin => time()) + (array)get_option('recently_activated'));
	}

	private static function is_gravityforms_installed(){
		return class_exists("RGForms");
	}

	private static function is_gravityforms_supported(){
		if(class_exists("GFCommon")){
			$is_correct_version = version_compare(GFCommon::$version, self::$min_gravityforms_version, ">=");
			return $is_correct_version;
		}
		else{
			return false;
		}
	}

	protected static function has_access($required_permission){
		$has_members_plugin = function_exists('members_get_capabilities');
		$has_access = $has_members_plugin ? current_user_can($required_permission) : current_user_can("level_7");
		if($has_access)
			return $has_members_plugin ? $required_permission : "level_7";
		else
			return false;
	}

	private static function get_customer_information($form, $config=null){
		$form_fields = self::get_form_fields($form);
		$str = "<table cellpadding='0' cellspacing='0'><tr><td class='oxapay_col_heading'>".__("OxaPay Feeds", "gravityformsoxapay")."</td><td class='oxapay_col_heading'>".__("Form Fields", "gravityformsoxapay")."</td></tr>";
		$customer_fields = self::get_customer_fields();
		foreach($customer_fields as $field){
			$selected_field = $config ? $config["meta"]["customer_fields"][$field["name"]] : "";
			$str .= "<tr><td class='oxapay_field_cell'>" . $field["label"] . "</td><td class='oxapay_field_cell'>" . self::get_mapped_field_list($field["name"], $selected_field, $form_fields) . "</td></tr>";
		}
		$str .= "</table>";

		return $str;
	}

	private static function get_customer_fields(){
		return array(
			array("name" => "first_name" , "label" => "First Name"),
			array("name" => "last_name" , "label" =>"Last Name"),
			array("name" => "email" , "label" =>"Email"),
			array("name" => "address1" , "label" =>"Address"),
			array("name" => "address2" , "label" =>"Address 2"),
			array("name" => "city" , "label" =>"City"),
			array("name" => "state" , "label" =>"State"),
			array("name" => "zip" , "label" =>"Zip"),
			array("name" => "country" , "label" =>"Country")
		);
	}

	private static function get_mapped_field_list($variable_name, $selected_field, $fields){
		$field_name = "oxapay_customer_field_" . $variable_name;
		$str = "<select name='$field_name' id='$field_name'><option value=''></option>";
		foreach($fields as $field){
			$field_id = $field[0];
			$field_label = esc_html(GFCommon::truncate_middle($field[1], 40));

			$selected = $field_id == $selected_field ? "selected='selected'" : "";
			$str .= "<option value='" . $field_id . "' ". $selected . ">" . $field_label . "</option>";
		}
		$str .= "</select>";
		return $str;
	}

	private static function get_product_options($form, $selected_field){
		$str = "<option value=''> ".__("Please Select", "gravityformsoxapay")." </option>";
		$fields = GFCommon::get_fields_by_type($form, array("product"));

		foreach($fields as $field){
			$field_id = $field["id"];
			$field_label = RGFormsModel::get_label($field);

			$selected = $field_id == $selected_field ? "selected='selected'" : "";
			$str .= "<option value='" . $field_id . "' ". $selected . ">" . $field_label . "</option>";
		}

		$selected = $selected_field == 'all' ? "selected='selected'" : "";
		$str .= "<option value='all' " . $selected . "> ".__("Total Form", "gravityformsoxapay")." </option>";

		return $str;
	}

	private static function get_form_fields($form){
		$fields = array();

		if(is_array($form["fields"])){
			foreach($form["fields"] as $field){
				if(isset($field["inputs"]) && is_array($field["inputs"])){

					foreach($field["inputs"] as $input)
						$fields[] = array($input["id"], GFCommon::get_label($field, $input["id"]));
				}
				else if(!rgar($field, 'displayOnly')){
					$fields[] = array($field["id"], GFCommon::get_label($field));
				}
			}
		}
		return $fields;
	}

	private static function return_url($form_id, $lead_id) {
		$pageURL = GFCommon::is_ssl() ? "https://" : "http://";

		if ($_SERVER["SERVER_PORT"] != "80")
			$pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$_SERVER["REQUEST_URI"];
		else
			$pageURL .= $_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];

		$ids_query = "ids={$form_id}|{$lead_id}";
		$ids_query .= "&hash=" . wp_hash($ids_query);

		$pageURL = remove_query_arg(array('do', 'gf_oxapay_return'), $pageURL);
		return add_query_arg("gf_oxapay_return", base64_encode($ids_query), $pageURL);
	}

	private static function is_oxapay_page(){
		$current_page = trim(strtolower(RGForms::get("page")));
		return in_array($current_page, array("gf_oxapay"));
	}

	protected static function get_base_url(){
		return plugins_url(null, __FILE__);
	}

	protected static function get_base_path(){
		$folder = basename(dirname(__FILE__));
		return WP_PLUGIN_DIR . "/" . $folder;
	}

	public static function admin_edit_payment_status($payment_status, $form_id, $lead)
	{
		$payment_gateway = gform_get_meta($lead["id"], "payment_gateway");
		require_once(self::get_base_path() . "/data.php");
		$oxapay_feed_id = gform_get_meta($lead["id"], "oxapay_feed_id");
		$feed_config = GFOxaPayData::get_feed($oxapay_feed_id);
		$transaction_type = rgars($feed_config, "meta/type");
		if ($payment_gateway <> "oxapay" || strtolower(rgpost("save")) <> "edit" || $payment_status == "Completed" || $transaction_type == "subscription")
			return $payment_status;

		$payment_string = gform_tooltip("oxapay_edit_payment_status","",true);
		$payment_string .= '<select id="payment_status" name="payment_status">';
		$payment_string .= '<option value="' . $payment_status . '" selected>' . $payment_status . '</option>';
		$payment_string .= '<option value="Completed">Completed</option>';
		$payment_string .= '</select>';
		return $payment_string;
	}
	public static function admin_edit_payment_status_details($form_id, $lead)
	{
		//check meta to see if this entry is oxapay
		$payment_gateway = gform_get_meta($lead["id"], "payment_gateway");
		$form_action = strtolower(rgpost("save"));
		if ($payment_gateway <> "oxapay" || $form_action <> "edit")
			return;

		//get data from entry to pre-populate fields
		$payment_amount = rgar($lead, "payment_amount");
		if (empty($payment_amount))
		{
			$form = RGFormsModel::get_form_meta($form_id);
			$payment_amount = GFCommon::get_order_total($form,$lead);
		}
		$transaction_id = rgar($lead, "transaction_id");
		$payment_date = rgar($lead, "payment_date");
		if (empty($payment_date))
		{
			$payment_date = gmdate("y-m-d H:i:s");
		}

		//display edit fields
		?>
		<div id="edit_payment_status_details" style="display:block">
			<table>
				<tr>
					<td colspan="2"><strong>Payment Information</strong></td>
				</tr>


				<tr>
					<td>Date:<?php gform_tooltip("oxapay_edit_payment_date") ?></td>
					<td><input type="text" id="payment_date" name="payment_date" value="<?php echo $payment_date?>"></td>
				</tr>
				<tr>
					<td>Amount:<?php gform_tooltip("oxapay_edit_payment_amount") ?></td>
					<td><input type="text" id="payment_amount" name="payment_amount" value="<?php echo $payment_amount?>"></td>
				</tr>
				<tr>
					<td nowrap>Transaction ID:<?php gform_tooltip("oxapay_edit_payment_transaction_id") ?></td>
					<td><input type="text" id="oxapay_transaction_id" name="oxapay_transaction_id" value="<?php echo $transaction_id?>"></td>
				</tr>
			</table>
		</div>
		<?php
	}

	public static function admin_update_payment($form, $lead_id) {
		check_admin_referer('gforms_save_entry', 'gforms_save_entry');
		$payment_gateway = gform_get_meta($lead_id, "payment_gateway");
		$form_action = strtolower(rgpost("save"));
		if ($payment_gateway <> "oxapay" || $form_action <> "update")
			return;
		$lead = RGFormsModel::get_lead($lead_id);
		$payment_status = rgpost("payment_status");
		if (empty($payment_status))
		{
			$payment_status = $lead["payment_status"];
		}

		$payment_amount = rgpost("payment_amount");
		$payment_transaction = rgpost("oxapay_transaction_id");
		$payment_date = rgpost("payment_date");
		if (empty($payment_date))
		{
			$payment_date = gmdate("y-m-d H:i:s");
		}
		else
		{
			//format date entered by user
			$payment_date = date("Y-m-d H:i:s", strtotime($payment_date));
		}

		global $current_user;
		$user_id = 0;
		$user_name = "System";
		if($current_user && $user_data = get_userdata($current_user->ID)){
			$user_id = $current_user->ID;
			$user_name = $user_data->display_name;
		}

		$lead["payment_status"] = $payment_status;
		$lead["payment_amount"] = $payment_amount;
		$lead["payment_date"] = $payment_date;
		$lead["transaction_id"] = $payment_transaction;

		if($payment_status == 'Completed' && !$lead["is_fulfilled"])
		{
			self::fulfill_order($lead, $payment_transaction, $payment_amount);
			$lead["is_fulfilled"] = true;
		}
		GFAPI::update_entry($entry); 
		RGFormsModel::add_note($lead["id"], $user_id, $user_name, sprintf(__("Transaction Updated . Status : %s .Amount : %s .TrackId : %s .Date : %s .", "gravityformsoxapay"), $lead["payment_status"], GFCommon::to_money($lead["payment_amount"], $lead["currency"]), $payment_transaction, $lead["payment_date"]));
	}

	function set_logging_supported($plugins)
	{
		$plugins[self::$slug] = "OxaPay Payments";
		return $plugins;
	}

	private static function log_error($message){
		if(class_exists("GFLogging"))
		{
			GFLogging::include_logger();
			GFLogging::log_message(self::$slug, $message, KLogger::ERROR);
		}
	}

	private static function log_debug($message){
		if(class_exists("GFLogging"))
		{
			GFLogging::include_logger();
			GFLogging::log_message(self::$slug, $message, KLogger::DEBUG);
		}
	}
}

if(!function_exists("rgget")){
function rgget($name, $array=null){
	if(!isset($array))
		$array = $_GET;

	if(isset($array[$name]))
		return $array[$name];

	return "";
}
}

if(!function_exists("rgpost")){
function rgpost($name, $do_stripslashes=true){
	if(isset($_POST[$name]))
		return $do_stripslashes ? stripslashes_deep($_POST[$name]) : $_POST[$name];

	return "";
}
}

if(!function_exists("rgar")){
function rgar($array, $name){
	if(isset($array[$name]))
		return $array[$name];

	return '';
}
}

if(!function_exists("rgars")){
function rgars($array, $name){
	$names = explode("/", $name);
	$val = $array;
	foreach($names as $current_name){
		$val = rgar($val, $current_name);
	}
	return $val;
}
}

if(!function_exists("rgempty")){
function rgempty($name, $array = null){
	if(!$array)
		$array = $_POST;

	$val = rgget($name, $array);
	return empty($val);
}
}

if(!function_exists("rgblank")){
function rgblank($text){
	return empty($text) && strval($text) != "0";
}
}

?>	